﻿using System;
using Telerik.Windows.Controls.Scheduling;

namespace GanttDragDependencies
{
    public class CustomDragDropBehavior : SchedulingDragDropBehavior
    {
        private DateTime lastDropTime;
        protected override bool CanStartDrag(SchedulingDragDropState state)
        {
            lastDropTime = state.DraggedItem.Start;

            return base.CanStartDrag(state);
        }
        protected override void Drop(SchedulingDragDropState state)
        {
            base.Drop(state);

            var timeToDrop = state.DestinationSlot.Start;
            var task = state.DraggedItem as CustomGanttTask;

            if (task != null)
            {
                this.UpdateProperties(timeToDrop, task);
            }

            lastDropTime = timeToDrop;
        }

        private void UpdateProperties(DateTime timeToDrop, CustomGanttTask task)
        {
            foreach (var dependantTask in task.DependantTasks)
            {
                var delta = timeToDrop.Subtract(lastDropTime);
                dependantTask.Start += delta;
                dependantTask.End += delta;

                if (dependantTask.DependantTasks.Count > 0)
                {
                    this.UpdateProperties(timeToDrop, dependantTask);
                }
            }
        }
    }
}
